/*
 * Main game iteration, and misc functions
 *
 * Copyright (c) 2000 Michael Pearson <alcaron@ozemail.com.au>
 *
 * This file is licensed under the GNU General Public License. See the file
 * "COPYING" for more details.
 */

#include <sys/time.h>
#include <unistd.h>

#include "breakout.h"
#include "gui.h"
#include "game.h"
#include "anim.h"
#include "block.h"
#include "ball.h"
#include "bat.h"
#include "powerup.h"
#include "flags.h"

#define NUM_LIVES 5

/* Framerate */
#define FRAMERATE (CLOCKS_PER_SEC / 50)
#define MSCPSDIFF (1000.0 / CLOCKS_PER_SEC)
#define NEWLIFESCORE 20000
#define NEXTLEVELSCORE 5000

void iterate_game(Game *game) {
	struct timeval start_tv, end_tv;
	struct timezone tz;
	gint32 diff_t;

	while(game->state == STATE_RUNNING) {
		gettimeofday(&start_tv, &tz);
		game->mouse_move = get_mouse_x_position();

		if(game->score > game->last_newlife_score + NEWLIFESCORE
				* game->flags.score_modifier) {
			game->last_newlife_score = game->score;
			new_life(game);
		}

		if(check_level_end(game)) {
			next_level(game);
		} else {
			iterate_bat(game);
			iterate_balls(game);
			iterate_powerups(game);
			iterate_blocks(game);
		}
		update_gnome(game);

		game->fire1_pressed = FALSE;
		game->fire2_pressed = FALSE;
		process_gnome_events();
		
		gettimeofday(&end_tv, &tz);
		diff_t = (start_tv.tv_sec * 1000000 + start_tv.tv_usec
				+ FRAMERATE) - (end_tv.tv_sec * 1000000
				+ end_tv.tv_usec);
		if(diff_t > 0)
			usleep(diff_t);
	}
}

/* Makes a new game, and starts it up */
void run_game(Game *game) {
	if(game->state != STATE_STOPPED) {
		gui_end_game(ENDGAME_MENU);
		end_game(game);
	}

	game->flags.difficulty = game->flags.next_game_difficulty;
	compute_flags(&(game->flags));
	game->state = STATE_RUNNING;
	game->balls = NULL;
	new_bat(game);
	new_ball_stuck(game);
	game->powerups = NULL;
	game->level = generate_level(1);
	game->score = 0;
	game->lives = NUM_LIVES;
	game->level_no = 1;
	game->fire1_pressed = FALSE;
	game->fire2_pressed = FALSE;
	game->mouse_move = 0;
	game->keyboard_move = 0;
	gui_begin_game();
	update_gnome(game);
	iterate_game(game);
}

/* If the game is running, pauses it. If the game is paused, resumes it.
 * If the game is stopped, does nothing. explicit is TRUE when the game was
 * paused/resumed from the menu. An explicit pause cannot be overrided by
 * the pause/resume of loss of focus or the preferences box */
void pause_game(Game *game, gboolean explicit) {
	if(explicit) {
		if(game->state == STATE_PAUSED) {
			if(game->paused_explicit) {
				game->state = STATE_RUNNING;
				game->paused_explicit = FALSE;
				iterate_game(game);
			} else {
				game->paused_explicit = TRUE;
			}
		} else if(game->state == STATE_RUNNING) {
			game->state = STATE_PAUSED;
			game->paused_explicit = TRUE;
		}
	} else if(game->paused_explicit == FALSE) {
		if(game->state == STATE_PAUSED) {
			if(explicit == game->paused_explicit) {
				game->state = STATE_RUNNING;
				iterate_game(game);
			}
		} else if(game->state == STATE_RUNNING) {
			game->state = STATE_PAUSED;
		}
	}
}

/* Ends the game and de-allocates memory. gui.c:gui_end_game must be called
 * BEFORE this function */
void end_game(Game *game) {
	if(game->state != STATE_STOPPED) {
		game->state = STATE_STOPPED;
		destroy_ball_list(game->balls);
		game->balls = NULL;
		destroy_powerup_list(game->powerups);
		game->powerups = NULL;
		destroy_bat(game);
		if(game->level)
			destroy_level(game);
		game->score = 0;
		game->lives = 0;
		game->level_no = 0;
		update_gnome(game);
	}
}

/* Lose a life. If none are left, then end the game */
void lose_life(Game *game) {
	reset_bat_type(game);
	game->lives--;
	if(game->lives != -1) {
		destroy_ball_list(game->balls);
		game->balls = NULL;
		new_ball_stuck(game);
	} else {
		gui_end_game(ENDGAME_LOSE);
		end_game(game);
	}
}

/* Goes to the next level */
void next_level(Game *game) {
	ADD_SCORE(game, NEXTLEVELSCORE);
	destroy_ball_list(game->balls);
	game->balls = NULL;
	destroy_powerup_list(game->powerups);
	game->powerups = NULL;
	destroy_level(game);
	game->level_no++;
	new_ball_stuck(game);
	game->level = generate_level(game->level_no);
	reset_bat_type(game);
	if(!game->level) {
		gui_end_game(ENDGAME_WIN);
		end_game(game);
	}
	update_gnome(game);
}
	
/* Key handlers. Called from gui.c */
void key_left_pressed(Game *game) {
	if(game->state == STATE_RUNNING)
		game->keyboard_move = -(game->flags.bat_speed);
}

void key_left_released(Game *game) {
	game->keyboard_move = 0;
}

void key_right_pressed(Game *game) {
	if(game->state == STATE_RUNNING)
		game->keyboard_move = game->flags.bat_speed;
}

void key_right_released(Game *game) {
	game->keyboard_move = 0;
}

void key_fire1_pressed(Game *game) {
	if(game->state == STATE_RUNNING)
		game->fire1_pressed = TRUE;
}

void key_fire1_released(Game *game) {
	return;
}

void key_fire2_pressed(Game *game) {
	if(game->state == STATE_RUNNING)
		game->fire2_pressed = TRUE;
	return;
}

void key_fire2_released(Game *game) {
	return;
}

void new_life(Game *game) {
	game->lives++;
}
