/*
 * Callbacks for events triggered by gui.c. Located here in the interests of
 * cleanliness.
 *
 * Every funciton here should take a GuiInfo * as its data element, unless
 * otherwise specified. Even if the function does not use it currently, it
 * should be passed anyway, incase it is used in the future.
 *
 * Copyright (c) 2000 Michael Pearson <alcaron@ozemail.com.au>
 *
 * This file is licensed under the GNU General Public License. See the file
 * "COPYING" for more details.
 */

#include "breakout.h"
#include "gui.h"
#include "gui-callbacks.h"
#include "gui-preferences.h"
#include "game.h"
#include "ball.h"

#define NEXTLEVEL_KEY 1

/* Whether we should unpause when we get focus back */
static gboolean unpause = FALSE;

/* We need this here because gnome-breakout tends to output alot of nasty
 * warning messages if we quit without killing off alot of canvas objects */
void cb_exit_game(GtkWidget *widget, gpointer data) {
	GuiInfo *gui;

        gui = (GuiInfo *) data;
	if(gui->game->state != STATE_STOPPED) {
		gui_end_game(ENDGAME_MENU);
        	end_game(gui->game);
	}

        gtk_object_destroy(GTK_OBJECT(gui->background));
        gtk_object_destroy(GTK_OBJECT(gui->title_image));

        gtk_main_quit();
}

/* And we need this because the event mechanism is different from the menu
 * mechanism */
gint cb_sig_exit_game(GtkWidget *widget, GdkEvent *event, gpointer data) {
	cb_exit_game(NULL, data);

	return FALSE;
}

/* Show the about box. I wonder, does the about box dialog free itself? */
void cb_show_about_box(GtkWidget *widget, gpointer data) {
        GtkWidget *dlg;
	GuiInfo *gui;

        const gchar *authors[] = { "Michael Pearson", NULL };
	const gchar comment[] = N_(
		"The classic arcade Breakout. Written for my cat, Marmalade."
		"\n \n"
		"This, if you haven't guessed yet, is a development release. Send bugreports and comments to alcaron@ozemail.com.au, but check to see if they haven't already been listed in TODO or BUGS."
		"\n \n"
		"Also, I've noticed a couple of segfaults that seem to happen randomly, and I'm having trouble reproducing them. So run this under gdb, and if it dies, send me the backtrace.");

        dlg = gnome_about_new(_("GNOME Breakout"), VERSION,
                        _("Copyright (c) 2000 Michael Pearson"),
                        authors,
			_(comment),
                        NULL);

	gui = (GuiInfo *) data;
        gnome_dialog_set_parent(GNOME_DIALOG(dlg), GTK_WINDOW(gui->app));
        gtk_widget_show(dlg);
}

/* Called when the user holds down a key. If it's a key we recognise, pass it
 * on to the relevant game.c function */
gint cb_keydown(GtkWidget *widget, GdkEventKey *event, gpointer data) {
	Game *game;
	game = ((GuiInfo *) data)->game;

	if(game->flags.keyboard_control) {
        	if(event->keyval == game->flags.left_key) {
                	key_left_pressed(game);
        	} else if(event->keyval == game->flags.right_key) {
                	key_right_pressed(game);
        	} else if(event->keyval == game->flags.fire1_key) {
			key_fire1_pressed(game);
		} else if(event->keyval == game->flags.fire2_key) {
			key_fire2_pressed(game);
		}
	}

        return FALSE;
}

/* Same as above, but for when a key is released */
gint cb_keyup(GtkWidget *widget, GdkEventKey *event, gpointer data) {
	Game *game;
	game = ((GuiInfo *) data)->game;

	if(game->flags.keyboard_control) {
        	if(event->keyval == game->flags.left_key) {
      	        	key_left_released(game);
	        } else if(event->keyval == game->flags.right_key) {
	                key_right_released(game);
       		} else if(event->keyval == game->flags.fire1_key) {
			key_fire1_released(game);
		} else if(event->keyval == game->flags.fire2_key) {
			key_fire2_released(game);
		}
	}
#ifdef NEXTLEVEL_KEY
#warning NEXTLEVEL_KEY cheat is enabled.
	if(event->keyval == GDK_L && game->state == STATE_RUNNING)
		next_level(game);
#endif

        return FALSE;
}

/* Hides the title image, and starts a new game */
void cb_new_game(GtkWidget *widget, gpointer data) {
	GuiInfo *gui;
	gui = (GuiInfo *) data;

	/* Hide the title */
        gnome_canvas_item_hide(gui->title_image);
        gnome_canvas_item_show(gui->background);
        gnome_canvas_update_now(gui->canvas);

	run_game(gui->game);
}

/* Ends the game and shows the title */
void cb_end_game(GtkWidget *widget, gpointer data) {
	GuiInfo *gui;
	gui = (GuiInfo *) data;

	gui_end_game(ENDGAME_MENU);
	end_game(gui->game);
}

/* Just calls pause_game. Included for consistency */
void cb_pause_game(GtkWidget *widget, gpointer data) {
	GuiInfo *gui;
	gui = (GuiInfo *) data;

	pause_game(gui->game, TRUE);
}

/* Calls make_preferences_box. Again, included for consistency */
void cb_preferences(GtkWidget *widget, gpointer data) {
	GuiInfo *gui;
	gui = (GuiInfo *) data;

	make_preferences_box(gui);
}

/* For now, just prints a pretty message */
void cb_help(GtkWidget *widget, gpointer data) {
	GuiInfo *gui;
	GtkWidget *mbox;

	gui = (GuiInfo *) data;
	mbox = gnome_ok_dialog_parented(_("Help hasn't been implimented yet. So go ahead and write some for me :)"), GTK_WINDOW(gui->app));
	gtk_widget_show(mbox);
}

/* The app has lost focus */
gboolean cb_focus_out(GtkWidget *widget, GdkEvent *event, gpointer data) {
	GuiInfo *gui;
	gui = (GuiInfo *) data;

	if(gui->game->flags.pause_on_focus_loss) {
		if(gui->game->state == STATE_RUNNING) {
			unpause = TRUE;
			pause_game(gui->game, FALSE);
		}
	}

	return FALSE;
}

/* The app has gained focus */
gboolean cb_focus_in(GtkWidget *widget, GdkEvent *event, gpointer data) {
	GuiInfo *gui;
	gui = (GuiInfo *) data;

	if(gui->game->state == STATE_PAUSED && unpause == TRUE) {
		unpause = FALSE;
		pause_game(gui->game, FALSE);
	}

	return FALSE;
}

gboolean cb_canvas_button_press(GtkWidget *widget, GdkEventButton *event,
		gpointer data) {
	GuiInfo *gui;
	gui = (GuiInfo *) data;

	if(gui->game->flags.mouse_control) {
		if(event->button == 1)
			key_fire1_pressed(gui->game);
		else if(event->button == 3)
			key_fire2_pressed(gui->game);
	}

	return FALSE;
}

/* Kills the current ball, if it gets stuck inside a block */
void cb_kill_ball(GtkWidget *widget, gpointer data) {
	GuiInfo *gui;
	gui = (GuiInfo *) data;

	if(gui->game->balls)
		ball_die(gui->game, (Ball *) gui->game->balls->data);
}

/* Displays the scores */
void cb_scores(GtkWidget *widget, gpointer data) {
	gnome_scores_display(_("GNOME Breakout: Scores"), PACKAGE, NULL, 0);
}
