/*
 * The functions which create and handle the preferences dialog.
 * This is a damn long file, so I've sectioned it off somewhat with comments.
 *
 * Copyright (c) 2000 Michael Pearson <alcaron@ozemail.com.au>
 *
 * This file is licensed under the GNU General Public License. See the file
 * "COPYING" for more details.
 */

#include "breakout.h"
#include "gui.h"
#include "gui-preferences.h"
#include "game.h"
#include "flags.h"

static GtkWidget *dialog = NULL;
static Flags newflags;
static gboolean unpause = FALSE;

/* Internal functions */
static void init_preferences_box(GuiInfo *gui);
static void init_game_page(GuiInfo *gui);
static void init_keybinding_page(GuiInfo *gui);

/* Callbacks. Don't think I'll move them to a different file here */
static void cb_pref_apply(GtkWidget *propertybox, gint page_num, gpointer data);
static void cb_pref_help(GtkWidget *propertybox, gint page_num, gpointer data);
static void cb_pref_destroy(GtkWidget *widget, gpointer data);
static void cb_key_left(GtkWidget *widget, GdkEventKey *event, gpointer data);
static void cb_key_right(GtkWidget *widget, GdkEventKey *event, gpointer data);
static void cb_key_fire1(GtkWidget *widget, GdkEventKey *event, gpointer data);
static void cb_key_fire2(GtkWidget *widget, GdkEventKey *event, gpointer data);
static void cb_diff(GtkWidget *widget, gpointer data);
static void cb_ctrl_key(GtkWidget *widget, gpointer data);
static void cb_ctrl_mouse(GtkWidget *widget, gpointer data);
static void cb_pofl(GtkWidget *widget, gpointer data);
static void cb_bat_speed(GtkWidget *widget, gpointer data);

void make_preferences_box(GuiInfo *gui) {

	/* Don't let the user run two preferences boxes at the same time */
	if(dialog) {
		gtk_widget_grab_focus(GTK_WIDGET(dialog));
		return;
	}

	if(gui->game->state == STATE_RUNNING) {
		pause_game(gui->game, FALSE);
		unpause = TRUE;
	} else {
		unpause = FALSE;
	}

	newflags = load_flags();

	init_preferences_box(gui);
	init_game_page(gui); 
	init_keybinding_page(gui);
	gtk_widget_show(dialog);
}

/* Apply has been clicked */
static void cb_pref_apply(GtkWidget *propertybox, gint page_num, gpointer data) {
	GuiInfo *gui;

	gui = (GuiInfo *) data;

	save_flags(newflags);

	gui->game->flags = newflags;
}

/* Help has been clicked */
static void cb_pref_help(GtkWidget *propertybox, gint page_num, gpointer data) {
	GuiInfo *gui;
	GtkWidget *mbox;

	gui = (GuiInfo *) data;
	mbox = gnome_ok_dialog_parented(_("Help hasn't been implimented yet. So go ahead and write some for me :)"), GTK_WINDOW(gui->app));
	gtk_widget_show(mbox);
}

/* Preferences dialog has been destroyed */
static void cb_pref_destroy(GtkWidget *widget, gpointer data) {
	GuiInfo *gui;
	gui = (GuiInfo *) data;

	dialog = NULL;

	if(unpause && gui->game->state == STATE_PAUSED)
		pause_game(gui->game, FALSE);

}

/*
 * Page generation
 */

/* Initialise the actual dialog box, but don't add anything */
void init_preferences_box(GuiInfo *gui) {
	dialog = gnome_property_box_new();
	gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(gui->app));
	gtk_window_set_title(GTK_WINDOW(dialog),
			_("GNOME Breakout Preferences"));

	gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
			GTK_SIGNAL_FUNC(cb_pref_destroy), gui);
	gtk_signal_connect(GTK_OBJECT(dialog), "apply",
			GTK_SIGNAL_FUNC(cb_pref_apply), gui);
	gtk_signal_connect(GTK_OBJECT(dialog), "help",
			GTK_SIGNAL_FUNC(cb_pref_help), gui);
	
}

/* Initialise the "game" section of the preferences box */
void init_game_page(GuiInfo *gui) {
	GtkWidget *gametablabel;
	GtkWidget *gamevbox;
	GtkWidget *gamehbox1;
	GtkWidget *gamehbox2;

	GtkWidget *bat_speed_label;
	GtkWidget *bat_speed_hbox;
	GtkObject *bat_speed_adjustment;
	GtkWidget *bat_speed_sbutton;
	GtkWidget *pause_on_focus_loss_check;

	GtkWidget *control_frame;
	GtkWidget *control_vbox;
	GtkWidget *control_mouse_radio;
	GtkWidget *control_keyboard_radio;
	
	GtkWidget *difficulty_frame;
	GtkWidget *difficulty_vbox;
	GtkWidget *difficulty_easy_radio;
	GtkWidget *difficulty_medium_radio;
	GtkWidget *difficulty_hard_radio;

	/* Vbox and Tablabel */
	gametablabel = gtk_label_new(_("Game"));
	gamevbox = gtk_vbox_new(FALSE, GNOME_PAD);
	gamehbox1 = gtk_hbox_new(FALSE, GNOME_PAD);
	gamehbox2 = gtk_hbox_new(FALSE, GNOME_PAD);
	gtk_box_pack_start(GTK_BOX(gamevbox), gamehbox1, TRUE, TRUE, GNOME_PAD);
	gtk_box_pack_start(GTK_BOX(gamevbox), gamehbox2, TRUE, TRUE, GNOME_PAD);
	gtk_widget_show(gametablabel);
	gtk_widget_show(gamevbox);
	gtk_widget_show(gamehbox1);
	gtk_widget_show(gamehbox2);

	/* Control Frame and Radio Buttons */
	control_frame = gtk_frame_new(_("Control"));
	gtk_frame_set_shadow_type(GTK_FRAME(control_frame), GTK_SHADOW_ETCHED_IN);
	control_vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(control_frame), control_vbox);
	control_keyboard_radio = gtk_radio_button_new_with_label(NULL, _("Keyboard"));
	control_mouse_radio = gtk_radio_button_new_with_label_from_widget(
			GTK_RADIO_BUTTON(control_keyboard_radio), _("Mouse"));
	gtk_box_pack_start_defaults(GTK_BOX(control_vbox),
			control_keyboard_radio);
	gtk_box_pack_start_defaults(GTK_BOX(control_vbox),
			control_mouse_radio);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(control_mouse_radio),
			newflags.mouse_control);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(control_keyboard_radio),
			newflags.keyboard_control);
	gtk_box_pack_start(GTK_BOX(gamehbox1), control_frame, TRUE, TRUE, GNOME_PAD);
	gtk_widget_show(control_frame);
	gtk_widget_show(control_vbox);
	gtk_widget_show(control_keyboard_radio);
	gtk_widget_show(control_mouse_radio);
	gtk_signal_connect(GTK_OBJECT(control_keyboard_radio), "toggled",
			GTK_SIGNAL_FUNC(cb_ctrl_key), NULL);
	gtk_signal_connect(GTK_OBJECT(control_mouse_radio), "toggled",
			GTK_SIGNAL_FUNC(cb_ctrl_mouse), NULL);

	/* Difficulty frame and radio buttons */
	if(gui->game->state == STATE_STOPPED)
		difficulty_frame = gtk_frame_new(_("Difficulty"));
	else
		difficulty_frame = gtk_frame_new(_("Difficulty (next game)"));

	gtk_frame_set_shadow_type(GTK_FRAME(difficulty_frame), GTK_SHADOW_ETCHED_IN);
	difficulty_vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(difficulty_frame), difficulty_vbox);
	difficulty_easy_radio = gtk_radio_button_new_with_label(NULL,
			_("Easy"));
	difficulty_medium_radio = gtk_radio_button_new_with_label_from_widget(
			GTK_RADIO_BUTTON(difficulty_easy_radio), _("Medium"));
	difficulty_hard_radio = gtk_radio_button_new_with_label_from_widget(
			GTK_RADIO_BUTTON(difficulty_easy_radio), _("Hard"));
	switch(newflags.next_game_difficulty) {
		case DIFFICULTY_EASY :
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(difficulty_easy_radio), TRUE);
			break;
		case DIFFICULTY_MEDIUM :
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(difficulty_medium_radio), TRUE);
			break;
		case DIFFICULTY_HARD :
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(difficulty_hard_radio), TRUE);
			break;
		default :
			g_assert_not_reached();
	}
	gtk_box_pack_start_defaults(GTK_BOX(difficulty_vbox),
			difficulty_easy_radio);
	gtk_box_pack_start_defaults(GTK_BOX(difficulty_vbox),
			difficulty_medium_radio);
	gtk_box_pack_start_defaults(GTK_BOX(difficulty_vbox),
			difficulty_hard_radio);
	gtk_box_pack_start(GTK_BOX(gamehbox1), difficulty_frame, TRUE, TRUE, GNOME_PAD);
	gtk_widget_show(difficulty_frame);
	gtk_widget_show(difficulty_vbox);
	gtk_widget_show(difficulty_easy_radio);
	gtk_widget_show(difficulty_medium_radio);
	gtk_widget_show(difficulty_hard_radio);
	gtk_signal_connect(GTK_OBJECT(difficulty_easy_radio), "toggled",
			GTK_SIGNAL_FUNC(cb_diff), (gpointer) DIFFICULTY_EASY);
	gtk_signal_connect(GTK_OBJECT(difficulty_medium_radio), "toggled",
			GTK_SIGNAL_FUNC(cb_diff), (gpointer) DIFFICULTY_MEDIUM);
	gtk_signal_connect(GTK_OBJECT(difficulty_hard_radio), "toggled",
			GTK_SIGNAL_FUNC(cb_diff), (gpointer) DIFFICULTY_HARD);

	/* Pause on focus loss */
	pause_on_focus_loss_check = gtk_check_button_new_with_label(_("Pause on loss of focus"));
	gtk_toggle_button_set_active(
			GTK_TOGGLE_BUTTON(pause_on_focus_loss_check),
			newflags.pause_on_focus_loss);
	gtk_box_pack_start(GTK_BOX(gamehbox2), pause_on_focus_loss_check,
			TRUE, FALSE, 0);
	gtk_widget_show(pause_on_focus_loss_check);
	gtk_signal_connect(GTK_OBJECT(pause_on_focus_loss_check), "toggled",
			GTK_SIGNAL_FUNC(cb_pofl), NULL);
	
	/* Bat speed */
	bat_speed_label = gtk_label_new(_("Bat speed: "));
	bat_speed_hbox = gtk_hbox_new(FALSE, GNOME_PAD);
	bat_speed_adjustment = gtk_adjustment_new((gfloat) newflags.bat_speed,
			(gfloat) MIN_BATSPEED, (gfloat) MAX_BATSPEED,
			1.0, 5.0, 1.0);
	bat_speed_sbutton = gtk_spin_button_new(
			GTK_ADJUSTMENT(bat_speed_adjustment), 1.0, 0);
	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(bat_speed_sbutton), 0);
	gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(bat_speed_sbutton), FALSE);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(bat_speed_sbutton), TRUE);
	gtk_box_pack_start_defaults(GTK_BOX(bat_speed_hbox), bat_speed_label);
	gtk_box_pack_start_defaults(GTK_BOX(bat_speed_hbox), bat_speed_sbutton);
	gtk_box_pack_start(GTK_BOX(gamehbox2), bat_speed_hbox, TRUE, FALSE, GNOME_PAD);
	gtk_widget_show(bat_speed_label);
	gtk_widget_show(bat_speed_hbox);
	gtk_widget_show(bat_speed_sbutton);
	gtk_signal_connect(GTK_OBJECT(bat_speed_adjustment), "value_changed",
			GTK_SIGNAL_FUNC(cb_bat_speed), bat_speed_sbutton);

	/* Add the game vbox to the dialog */
	gnome_property_box_append_page(GNOME_PROPERTY_BOX(dialog),
			gamevbox, gametablabel);
}

static void init_keybinding_page(GuiInfo *gui) {
	GtkWidget *keyvbox;
	GtkWidget *keyhbox1;
	GtkWidget *keyhbox2;
	GtkWidget *keytablabel;

	GtkWidget *left_hbox;
	GtkWidget *left_label;
	GtkWidget *left_entry;

	GtkWidget *right_hbox;
	GtkWidget *right_label;
	GtkWidget *right_entry;

	GtkWidget *fire1_hbox;
	GtkWidget *fire1_label;
	GtkWidget *fire1_entry;

	GtkWidget *fire2_hbox;
	GtkWidget *fire2_label;
	GtkWidget *fire2_entry;

	/* Init the vbox */
	keyvbox = gtk_vbox_new(TRUE, GNOME_PAD);
	keyhbox1 = gtk_hbox_new(TRUE, GNOME_PAD);
	keyhbox2 = gtk_hbox_new(TRUE, GNOME_PAD);
	keytablabel = gtk_label_new(_("Keybindings"));
	gtk_box_pack_start_defaults(GTK_BOX(keyvbox), keyhbox1);
	gtk_box_pack_start_defaults(GTK_BOX(keyvbox), keyhbox2);
	gtk_widget_show(keyvbox);
	gtk_widget_show(keyhbox1);
	gtk_widget_show(keyhbox2);

	/* Left */
	left_hbox = gtk_hbox_new(FALSE, GNOME_PAD);
	left_label = gtk_label_new(_("Left:"));
	gtk_misc_set_alignment(GTK_MISC(left_label), 0.0, 0.5);
	left_entry = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(left_entry), FALSE);
	gtk_entry_set_text(GTK_ENTRY(left_entry),
			gdk_keyval_name(newflags.left_key));
	gtk_box_pack_start_defaults(GTK_BOX(left_hbox), left_label);
	gtk_box_pack_start_defaults(GTK_BOX(left_hbox), left_entry);
	gtk_box_pack_start(GTK_BOX(keyhbox1), left_hbox, TRUE, TRUE, GNOME_PAD);
	gtk_widget_show(left_label);
	gtk_widget_show(left_entry);
	gtk_widget_show(left_hbox);
	gtk_signal_connect(GTK_OBJECT(left_entry), "key_press_event",
			GTK_SIGNAL_FUNC(cb_key_left), NULL);

	/* Right */
	right_hbox = gtk_hbox_new(FALSE, GNOME_PAD);
	right_label = gtk_label_new(_("Right:"));
	gtk_misc_set_alignment(GTK_MISC(right_label), 0.0, 0.5);
	right_entry = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(right_entry), FALSE);
	gtk_entry_set_text(GTK_ENTRY(right_entry),
			gdk_keyval_name(newflags.right_key));
	gtk_box_pack_start_defaults(GTK_BOX(right_hbox), right_label);
	gtk_box_pack_start_defaults(GTK_BOX(right_hbox), right_entry);
	gtk_box_pack_start(GTK_BOX(keyhbox2), right_hbox, TRUE, TRUE, GNOME_PAD);
	gtk_widget_show(right_label);
	gtk_widget_show(right_entry);
	gtk_widget_show(right_hbox);
	gtk_signal_connect(GTK_OBJECT(right_entry), "key_press_event",
			GTK_SIGNAL_FUNC(cb_key_right), NULL);

	/* Fire1 */
	fire1_hbox = gtk_hbox_new(FALSE, GNOME_PAD);
	fire1_label = gtk_label_new(_("Fire One:"));
	gtk_misc_set_alignment(GTK_MISC(fire1_label), 0.0, 0.5);
	fire1_entry = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(fire1_entry), FALSE);
	gtk_entry_set_text(GTK_ENTRY(fire1_entry),
			gdk_keyval_name(newflags.fire1_key));
	gtk_box_pack_start_defaults(GTK_BOX(fire1_hbox), fire1_label);
	gtk_box_pack_start_defaults(GTK_BOX(fire1_hbox), fire1_entry);
	gtk_box_pack_start(GTK_BOX(keyhbox1), fire1_hbox, TRUE, TRUE, GNOME_PAD);
	gtk_widget_show(fire1_label);
	gtk_widget_show(fire1_entry);
	gtk_widget_show(fire1_hbox);
	gtk_signal_connect(GTK_OBJECT(fire1_entry), "key_press_event",
			GTK_SIGNAL_FUNC(cb_key_fire1), NULL);

	/* Fire2 */
	fire2_hbox = gtk_hbox_new(FALSE, GNOME_PAD);
	fire2_label = gtk_label_new(_("Fire Two:"));
	gtk_misc_set_alignment(GTK_MISC(fire2_label), 0.0, 0.5);
	fire2_entry = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(fire2_entry), FALSE);
	gtk_entry_set_text(GTK_ENTRY(fire2_entry),
			gdk_keyval_name(newflags.fire2_key));
	gtk_box_pack_start_defaults(GTK_BOX(fire2_hbox), fire2_label);
	gtk_box_pack_start_defaults(GTK_BOX(fire2_hbox), fire2_entry);
	gtk_box_pack_start(GTK_BOX(keyhbox2), fire2_hbox, TRUE, TRUE, GNOME_PAD);
	gtk_widget_show(fire2_label);
	gtk_widget_show(fire2_entry);
	gtk_widget_show(fire2_hbox);
	gtk_signal_connect(GTK_OBJECT(fire2_entry), "key_press_event",
			GTK_SIGNAL_FUNC(cb_key_fire2), NULL);

	/* Add it to the dialog */
	gnome_property_box_append_page(GNOME_PROPERTY_BOX(dialog),
			keyvbox, keytablabel);
}

/*
 * Keybinding callbacks
 */
static void cb_key_left(GtkWidget *widget, GdkEventKey *event, gpointer data) {
	if(!dialog)
		return;

	gtk_entry_set_text(GTK_ENTRY(widget), gdk_keyval_name(event->keyval));
	newflags.left_key = event->keyval;

	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
}

static void cb_key_right(GtkWidget *widget, GdkEventKey *event, gpointer data) {
	if(!dialog)
		return;

	gtk_entry_set_text(GTK_ENTRY(widget), gdk_keyval_name(event->keyval));
	newflags.right_key = event->keyval;

	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
}

static void cb_key_fire1(GtkWidget *widget, GdkEventKey *event, gpointer data) {
	if(!dialog)
		return;

	gtk_entry_set_text(GTK_ENTRY(widget), gdk_keyval_name(event->keyval));
	newflags.fire1_key = event->keyval;

	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
}

static void cb_key_fire2(GtkWidget *widget, GdkEventKey *event, gpointer data) {
	if(!dialog)
		return;

	gtk_entry_set_text(GTK_ENTRY(widget), gdk_keyval_name(event->keyval));
	newflags.fire2_key = event->keyval;

	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
}

/*
 * Difficulty callbacks
 */
static void cb_diff(GtkWidget *widget, gpointer data) {
	newflags.next_game_difficulty = (Difficulty) data;
	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
}

/*
 * Control callbacks
 */
static void cb_ctrl_key(GtkWidget *widget, gpointer data) {
	newflags.mouse_control = FALSE;
	newflags.keyboard_control = TRUE;
	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
}
static void cb_ctrl_mouse(GtkWidget *widget, gpointer data) {
	newflags.mouse_control = TRUE;
	newflags.keyboard_control = FALSE;
	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
}

/*
 * Other option callbacks
 */
static void cb_pofl(GtkWidget *widget, gpointer data) {
	newflags.pause_on_focus_loss = gtk_toggle_button_get_active(
			GTK_TOGGLE_BUTTON(widget));
	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
}

static void cb_bat_speed(GtkWidget *widget, gpointer data) {
	newflags.bat_speed = gtk_spin_button_get_value_as_int(
			GTK_SPIN_BUTTON(data));
	gnome_property_box_changed(GNOME_PROPERTY_BOX (dialog));
}
